/*!
 * THIS FILE HAS BEEN GENERATED BY AN AUTOMATED TOOL.
 * DO NOT MODIFY DIRECTLY. INSTEAD, MODIFY THE APPROPRIATE SOURCE CODE.
 *
 * test v1.0.0 (dev)
 * Example project for Client.
 * Build Date: 2016-04-05
 */

/**
* @fileOverview NotificationTags module definition
*/

define(function(require) {
    'use strict';

    var Util = require('lib/Util');
    var push = window.PushNotification;

    var NotificationTags = {
        isPushEnabledKey: 'isPushEnabledKey',
        notificationTagsKey: 'notificationTags',

        init: function () {
        	try {
				this.$window = $(window);
				this.apiUrl = SETTINGS.API_URL;
			
				Util.log('setting up notification handlers');
				this.setUpHandlers();

				Util.log('registering for notification types');
				// Register for notification types on iOS
				push.registerForNotificationTypes(
					push.notificationType.badge |
					push.notificationType.sound |
					push.notificationType.alert
				);

				Util.log('enabling badges');
				// turn on badges for iOS
				push.setAutobadgeEnabled(true, function(message) {
					Util.log('badges enabled:', message);
				});

				Util.log('setting clear-on-resume hanlder');
				// clear badge count on resume
				document.addEventListener('resume', this.onResume.bind(this), false);

				// if push is enabled, we need to register the device
				if (this.getIsPushEnabled()) {
					Util.log('enabling push notifications');
					push.enablePush();
				} else {
					Util.log('disabling push notifications');
					push.disablePush();
				}
			} catch(e) {
				Util.log('error initializing NotificationTags: ' + e.message);
			}

            return this;
        },
        setUpHandlers: function() {
        	Util.log('setUpHandlers');
            this.onSetNotificationTagsSuccessHandler = this.onSetNotificationTagsSuccess.bind(this);
            this.onSetNotificationTagsFailHandler = this.onSetNotificationTagsFail.bind(this);
            this.onGetIncomingHandler = this.onGetIncoming.bind(this);
            return this;
        },

        onResume: function () {
            Util.log('onResume');
            push.resetBadge(); // reset badges
            push.getIncoming(this.onGetIncomingHandler);
        },

        getIsPushEnabled: function() {
        	Util.log('getIsPushEnabled');
        	var platform = device.platform.toLowerCase();
        	Util.log('platform=', platform);
        	
        	if(platform == 'ios') return 1;
        	
            var value = window.localStorage.getItem(this.isPushEnabledKey);
            if (value == null) {
            	Util.log('value is null');
                return value;
            }

            var result = parseInt(value, 10);
            Util.log('result=', result);
            return result;
        },

        setIsPushEnabled: function(isEnabled) {
        	try {
				Util.log('setIsPushEnabled, isEnabled=', isEnabled);
				window.localStorage.setItem(this.isPushEnabledKey,isEnabled);

				if (isEnabled) {
					push.enablePush();
				} else {
					push.disablePush();
				}

				push.isPushEnabled(function(hasPush) {
					if (hasPush) {
						Util.log('push enabled!');
					} else {
						Util.log('push not enabled!');
					}
				});
			} catch(e) {
				Util.log('error setting push enabled: ' + e.message);
			}
        },

        //returns {JQuery promise}
        getTags: function () {
        	Util.log('getTags');
            var dfd = $.Deferred();

            var tags = JSON.parse(window.localStorage.getItem(this.notificationTagsKey));
            if (!tags || tags.length === 0) {
                this.setNotificationTags().then(dfd.resolve);
            } else {
                dfd.resolve(tags);
            }

            return dfd.promise();
        },

        setTags: function (tags) {
            Util.log('setTags');
            window.localStorage.setItem(this.notificationTagsKey, JSON.stringify(tags));

            this._setTagsForUrbanAirship(tags);
        },

        _setTagsForUrbanAirship: function(tags) {
        	Util.log('_setTagsForUrbanAirship');
            var tagNames = [];
            for (var prop in tags) {
                if (!tags[prop].isChecked) {
                    continue;
                }
                tagNames.push(tags[prop].name);
            }
			
			if($.inArray(SETTINGS.FACILITY_ID.toString(), tagNames) < 0)
			{
				tagNames.push(SETTINGS.FACILITY_ID.toString());
			}
			
			tagNames.forEach(function(tag) {
				Util.log('Tag:', tag);
			});

            push.setTags(tagNames);
        },

        toggleTag: function(id, toggle) {
        	Util.log('toggleTag ' + id);
            this.getTags().then(function(tags) {
                for (var prop in tags) {
                    if (tags[prop].id == id) {
                        tags[prop].isChecked = toggle;
                        break;
                    }
                }

                this.setTags(tags);
            }.bind(this));
        },

        toggleAllTags: function(toggle) {
        	Util.log('toggleAllTags');
            this.getTags().then(function(tags) {
                for (var prop in tags) {
                    tags[prop].isChecked = toggle;
                }

                this.setTags(tags);
            }.bind(this));
        },

        logTags: function() {
            push.getTags(function(obj) {
                obj.tags.forEach(function(tag) {
                    console.log('Tag: ' + tag);
                });
            });
        },
        /**
         * Sets up the push notification tags
         *
         * returns {jquery promise}
         */
        setNotificationTags: function() {
        	Util.log('setNotificationTags');
            var data = {
                'p': SETTINGS.FACILITY_ID,
                'method': 'get_tags'
            };

            this.setTagsDeferred = $.Deferred();

            var jqxhr = $.ajax({
                url: this.apiUrl,
                data: data,
                type: 'GET'
            })
            .done(this.onSetNotificationTagsSuccessHandler)
            .fail(this.onSetNotificationTagsFailHandler);

            return this.setTagsDeferred.promise();
        },

        /**
         * Event handler for when the notification tags setting has completed
         *
         * @param {object} data The data returned from the ajax call to get tags
         */
        onSetNotificationTagsSuccess: function(data) {
        	Util.log('onSetNotificationTagsSuccess');
            var jsonParsed = Util.parseAjaxJson(data);
            var notificationTags = [];
            var notificationTag = null;
            for (var prop in jsonParsed) {
                notificationTag = {
                    'id': 'tag-' + prop,
                    'name': jsonParsed[prop].name,
                    'isChecked': true
                };
                Util.log('adding tag, id=tag-' + prop + ', name=' + jsonParsed[prop].name + ', isChecked=true');
                notificationTags.push(notificationTag);
            }
            this.setTags(notificationTags);
            this.setTagsDeferred.resolve(notificationTags);
        },

        /**
         * Event handler for when the notification tags setting has failed
         *
         * @param {object} error The error object returned
         */
        onSetNotificationTagsFail: function(error) {
        	Util.log('onSetNotificationTagsFail');
            navigator.notification.alert(
                'Could not set push notification tags',
                function() {},
                'Error',
                'Close'
            );
            Util.log('onSetNotificationTagsFail Error', error);
            this.setTagsDeferred.resolve([]);
        },

        onGetIncoming: function(incoming) {
        	Util.log('onGetIncoming');
            if (incoming && incoming.message.length > 0) {
                this.$window.trigger('pushNotificationReceived', [incoming]);
            }
        }
    }

    return NotificationTags.init();
});
