/*!
 * THIS FILE HAS BEEN GENERATED BY AN AUTOMATED TOOL.
 * DO NOT MODIFY DIRECTLY. INSTEAD, MODIFY THE APPROPRIATE SOURCE CODE.
 *
 * test v1.0.0 (dev)
 * Example project for Client.
 * Build Date: 2016-04-05
 */

/**
 * @fileOverview Util module definition
 */

define(function(require) {
    'use strict';

	var APP_UUID_KEY = 'AppUUID';
    var $ = require('jquery');

    /**
     * Static class containing commonly-used utility functions
     *
     * @name Util
     * @class Util Static class containing commonly-used utility functions
     * @constructor
     */
    var Util = {};

    /**
    * Logs information to the browser console (if console is available)
    *
    * @param {dynamic} Dynamic number of arguments to log
    */
    Util.log = function() {
        if (!SETTINGS.LOG_CONSOLE) return;

        if (typeof console !== 'undefined') {
            if (arguments.length === 1) { // Handle trivial case for just one argument
                console.log(arguments[0]);
            } else {
                if (console.log.apply) {  // Multi-argument logging
                    console.log.apply(console, arguments);
                } else { // Multi-argument logging not supported in IE, so we log an array instead
                    var argsArray = Array.prototype.slice.call(arguments, 0);
                    console.log(argsArray);
                }
            }
        }
    };

    /**
    * Converts an object to its string representation
    * Useful for logging in browsers that don't automatically make this conversion (eg, IE)
    *
    * @param {object} object The object to convert
    * @param {string} sep The separator character to use between each field (optional)
    * @returns {string} The string representation of object
    */
    Util.objectToString = function(object, sep) {
        if (sep == null) {
            sep = ', ';
        }

        var str = '';
        for (var i in object) {
            if (object.hasOwnProperty(i) && typeof(object[i]) !== 'function') {
                if (str != '') str += sep;
                str += i + ': ' + object[i];
            }
        }

        return str;
    };

    /**
    * Converts a string to numbers only
    *
    * @param {string} string The string to convert
    * @returns {string} The string representation of all numbers
    */
    Util.stringToNumber = function(string) {
        return string.match(/\d+/g).join('');
    };

    /**
    * Assures that http is at the front of a url
    *
    * @param {string} url The string url
    * @returns {string} The updated url with prepending http://
    */
    Util.assureHttp = function(url) {
        if (url == null || url == '') {
            return url;
        }

        var http = url.indexOf('http://');
        var https = url.indexOf('https://');

        if (http == -1 && https == -1) {
            url = 'http://' + url;
        }

        return url;
    };

    Util.windowOpen = function(url, name, specs, replace) {
        if (url == null || url == '') {
            navigator.notification.alert(
                'Sorry, no information is available at this time.',
                function() {},
                'Error',
                'Close'
            );
            return;
        }
        window.open(url, name, specs, replace);
    };

    /**
    * Used to parse data returned from an ajax call
    * Counteracts jquery.xdomainajax.js
    *
    * @param {object} data The returned data object from an ajax call
    * @returns {object} The parsed json object
    */
    Util.parseAjaxJson = function (data) {
        if (data.results instanceof Array) {
            // data has been wrapped by jquery.xdomainajax.js
            data = $.trim($(data.results[0]).text()).replace(/\n/g, ' ');
        }

        var jsonParsed = null;

        try {
            jsonParsed = $.parseJSON(data);
        }
        catch (err) {
            Util.log('Error in returned json from api.', err, data);
            navigator.notification.alert(
                'Error in returned json from api.',
                function () {
                    Util.log('Reloading...');
                    window.location = 'index.html';
                },
                'Error',
                'Reload Application'
            );

            //go to error page
            $("#ErrorPage").show();
        }

        return jsonParsed;
    };
    
    Util.appUUID = function() {
    	if('00000000-0000-0000-0000-000000000000' == SETTINGS.APP_UUID) {
    		SETTINGS.APP_UUID = window.localStorage.getItem(APP_UUID_KEY);
    		
    		if(null == SETTINGS.APP_UUID) {
    			SETTINGS.APP_UUID = Util.createUUID();
    			window.localStorage.setItem(APP_UUID_KEY, SETTINGS.APP_UUID);
    		}
    	}
    	
    	return SETTINGS.APP_UUID;
    }
	
	Util.createUUID = function() {
		// http://www.ietf.org/rfc/rfc4122.txt
		var s = [];
		var hexDigits = "0123456789abcdef";
		for (var i = 0; i < 36; i++) {
			s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
		}
		s[14] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
		s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01
		s[8] = s[13] = s[18] = s[23] = "-";

		var uuid = s.join("");
		return uuid;
	}

    return Util;
});
