/*!
 * THIS FILE HAS BEEN GENERATED BY AN AUTOMATED TOOL.
 * DO NOT MODIFY DIRECTLY. INSTEAD, MODIFY THE APPROPRIATE SOURCE CODE.
 *
 * test v1.0.0 (dev)
 * Example project for Client.
 * Build Date: 2016-04-05
 */

/**
 * @fileOverview SetPreferredLocationModel module definition
 */

define(function(require) {
    'use strict';

    var PREFERRED_LOCATION_KEY = 'preferredLocation';
    var BaseViewModel = require('view-models/BaseViewModel');
    var $ = require('jquery');
    var List = require('List');

    /**
     * A handler to load data to a view template.
     *
     * @name SetPreferredLocationModel
     * @class SetPreferredLocationModel A handler to load data to a view template.
     * @constructor
     * @param {object} data The data object returned from the backend API
     * @param {object} routeArgs The query object passed to the view via the route
     */
    var SetPreferredLocationModel = function (data, routeArgs) {
        if (data === null &&
            routeArgs === null) {
            return;
        }

        this.data = data;
        this.routeArgs = routeArgs;

        this.init();
    };


    /**
     * Added to make this class extend BaseViewModel
     */
    SetPreferredLocationModel.prototype = new BaseViewModel();
    SetPreferredLocationModel.prototype.constructor = SetPreferredLocationModel;

    var proto = SetPreferredLocationModel.prototype;

    proto.preferredLocation = {
        get: function () {
            if (SETTINGS.FACILITY_ID === 0) {
                SETTINGS.FACILITY_ID = Number(window.localStorage.getItem(PREFERRED_LOCATION_KEY) || 0);
            }
            return SETTINGS.FACILITY_ID;
        },
        set: function (value) {
            var facilityId = Number(value || 0);
            window.localStorage.setItem(PREFERRED_LOCATION_KEY, facilityId);
            SETTINGS.FACILITY_ID = facilityId;
            return value;
        },
        isSet: function() {
            return this.get() > 0;
        }
    }

    /**
     * Sets up the viewModel
     *
     * returns {jquery promise}
     */
    proto.getData = function () {
        //reset to dummy data for testing
        //this.data = [
        //    { id: '1945', prof_name: 'Reach Minneapolis', address1: '2345 Happy Ln N', address2: 'Minneapolis, MN 55441', lat: 44.983690, 'long': -93.268377 },
        //    { id: '18', prof_name: 'Reach St. Paul', address1: '4674 Magical Ave S', address2: 'St. Paul, MN 55119', lat: 44.953862, 'long': -93.090261 },
        //    { id: '1587', prof_name: 'Reach Bloomington', address1: '234 Yep', address2: 'Bloomington, MN 55423', lat: 44.844819, 'long': -93.298943 },
        //    { id: '45', prof_name: 'Reach Texas', address1: '345 good street', address2: 'Cowboytown, TX 98768', lat: 31.553965, 'long': -99.175964 },
        //    { id: '3', prof_name: 'Reach Florida', address1: 'some address1', address2: 'yep', lat: 27.747478, 'long': -81.514901},
        //    { id: '37', prof_name: 'Reach Illinois', address1: 'some address1', address2: 'yep', lat: 40.250025, 'long': -89.514494 },
        //    { id: '4', prof_name: 'Reach Madison', address1: 'some other address1', address2: 'yep', lat: 43.073333, 'long': -89.401271 }
        //];

        this.data = new List(this.data);

        //if preferredLocationId set, find in data, indicate that this is currently set 
        //note that they pass back id as a string
        var preferredLocation = this.preferredLocation.get().toString();

        if (preferredLocation > 0) {
            var selectedLocation = this.data.first(function (location) { return location.id === preferredLocation; });
            if (selectedLocation) {
                selectedLocation.selected = true;
            }
        }

        //try to get gps, so we can get nearest locations
        //TODO: consider putting all locations on page right away, then indicate that you're searching for location...then reset the data on the iScroll if we get it
        var dfd = $.Deferred();
        this.getCurrentPosition()
            .done(this.getCurrentPositionSuccess())
            .fail(this.getCurrentPositionError())
            .always(function () {
                dfd.resolve();
            });

        return dfd.promise();
    };

    /*
     * returns {jquery promise}
     */
    proto.getCurrentPosition = function () {
        var dfd = $.Deferred();
        //https://cordova.apache.org/docs/en/3.0.0/cordova_geolocation_geolocation.md.html#geolocationOptions
        navigator.geolocation.getCurrentPosition(onSuccess, onError, { maximumAge: 3000, timeout: 4000, enableHighAccuracy: true });
        function onSuccess(position) {
            dfd.resolve(position);
        }
        function onError(error) {
            dfd.reject(error);
        }
        return dfd.promise();
    };

    /*
     * returns {function} that is bound to this
     */
    proto.getCurrentPositionSuccess = function () {
        return function (position) {
            var gpsLatLon = { lat: position.coords.latitude, lon: position.coords.longitude };
            //take closest 15
            this.data
                .sortBy(function(location) {
                    var latDiff = Math.abs(location.lat - gpsLatLon.lat);
                    var lonDiff = Math.abs(location.long - gpsLatLon.lon);
                    return latDiff + lonDiff;
                })
                .take(15);

            this._getData();
        }.bind(this);
    };

    /*
     * returns function bound to this
     */
    proto.getCurrentPositionError = function() {
        return function (error) {
            //navigator.notification.alert(error.message);

            navigator.notification.alert('We were unable to determine your current location to find nearby facilities.');

            //fallback is to order by name
            this.data = this.data.sortBy('name');
            this._getData();
        }.bind(this);
    };

    /*
     * sets up viewModel. called after we're done sorting and filtering the locations
     */
    proto._getData = function () {
        var incomingData = this.data.toArray();

        this.data = {
            'section': []
        };

        var prop = null;
        var newRow = null;
        var data = null;
        var rows = null;

        var section = {
            'header': [],
            'rows': []
        };
        var sectionLength = this.data.section.push(section);

        for (prop in incomingData) {
            data = incomingData[prop];

            rows = this.data.section[sectionLength - 1].rows;
            newRow = {
                'location': data
            };
            rows.push(newRow);
        }

        this.data.title = 'Set Preferred Location';

        this.viewModel = this.data;
    };

    return SetPreferredLocationModel;
});
