/*!
 * THIS FILE HAS BEEN GENERATED BY AN AUTOMATED TOOL.
 * DO NOT MODIFY DIRECTLY. INSTEAD, MODIFY THE APPROPRIATE SOURCE CODE.
 *
 * test v1.0.0 (dev)
 * Example project for Client.
 * Build Date: 2016-04-05
 */

/**
 * @fileOverview BaseView module definition
 */

define(function(require) {
    'use strict';

    var Util = require('lib/Util');
    var $ = require('jquery');
    var EventBus = require('EventBus');
    //require('jquery-ui');

    var features = require('models/FeatureAdapter');
    var googleDocsPdfViewerUrl = 'http://docs.google.com/viewer?url=';

    var SetPreferredLocationModel = require('view-models/SetPreferredLocationModel');

    /**
     * A base view script.
     *
     * @name BaseView
     * @class BaseView A base view script.
     * @constructor
     * @param {jQuery} $element A reference to the containing DOM element
     */
    var BaseView = function($element) {
        if ($element === undefined) {
           return;
        }

        /**
         * A reference to the containing DOM element
         *
         * @default null
         * @name $element
         * @type {jQuery}
         */
        this.$element = $element;

        /**
         * Tracks whether component is enabled.
         *
         * @default false
         * @name isEnabled
         * @type {bool}
         */
        this.isEnabled = false;

        this.init();
    };

    BaseView.prototype = new EventBus();
    BaseView.prototype.constructor = BaseView;

    /**
     * Initializes the UI Component View
     * Runs a single setupHandlers call, followed by createChildren and layout
     * Exits early if it is already initialized
     *
     * returns {BaseView}
     */
    BaseView.prototype.init = function() {
        EventBus.call(this);

        this.setupHandlers()
           .createChildren()
           .layout()
           .enable()
           .viewIsReady();

        return this;
    };

    /**
     * Binds the scope of any handler functions
     * Should only be run on initialization of the view
     *
     * returns {BaseView}
     */
    BaseView.prototype.setupHandlers = function() {
        this.onInAppBrowserClickHandler = this.onInAppBrowserClick.bind(this);
        this.onPdfClickHandler = this.onPdfClick.bind(this);
        this.onPhoneClickHandler = this.onPhoneClick.bind(this);
        this.onMapClickHandler = this.onMapClick.bind(this);
        this.onEmailClickHandler = this.onEmailClick.bind(this);
        //filling in for jquery-mobile
        this.onBackClickHandler = this.onBackClick.bind(this);


        this.optionsOnRecieveDataCallbackHandler =
            this.optionsOnRecieveDataCallback.bind(this);

        return this;
    };

    /**
     * Create any child objects or references to DOM elements
     * Should only be run on initialization of the view
     *
     * @returns {BaseView}
     */
    BaseView.prototype.createChildren = function() {
        this.$inAppBrowserButton = this.$element.find('.js-in-app-browser');
        this.$pdfButton = this.$element.find('.js-pdf');
        this.$phoneButton = this.$element.find('.js-phone');
        this.$addressButton = this.$element.find('.js-address');
        this.$emailButton = this.$element.find('.js-email');

        //filling in for jquery-mobile
        this.$backbutton = this.$element.find('.btn_back');

        this.$optionsPartial = $('#options-partial');

        return this;
    };

    /**
     * Remove any child objects or references to DOM elements
     *
     * @returns {BaseView}
     */
    BaseView.prototype.removeChildren = function() {
        this.$inAppBrowserButton = null;
        this.$pdfButton = null;
        this.$phoneButton = null;
        this.$addressButton = null;
        this.$emailButton = null;

        //filling in for jquery-mobile
        this.$backbutton = null;

        this.$optionsPartial = null;

        return this;
    };

    /**
     * Performs measurements and applys any positioning style logic
     * Should be run anytime the parent layout changes
     *
     * @returns {BaseView}
     */
    BaseView.prototype.layout = function () {
        return this;
    };

    /**
     * Enables the component
     * Performs any event binding to handlers
     * Exits early if it is already enabled
     *
     * @returns {BaseView}
     */
    BaseView.prototype.enable = function() {
        if (this.isEnabled) {
            return this;
        }
        this.isEnabled = true;

        this.$pdfButton.on(
            features.TAP,
            this.onPdfClickHandler
        );
        this.$inAppBrowserButton.on(
            features.TAP,
            this.onInAppBrowserClickHandler
        );
        this.$phoneButton.on(
            features.TAP,
            this.onPhoneClickHandler
        );
        this.$addressButton.on(
            features.TAP,
            this.onMapClickHandler
        );
        this.$emailButton.on(
            features.TAP,
            this.onEmailClickHandler
        );
        //filling in for jquery-mobile
        this.$backbutton.on(
            features.TAP,
            this.onBackClickHandler
        );

        return this;
    };

    /**
     * Disables the component
     * Tears down any event binding to handlers
     * Exits early if it is already disabled
     *
     * @returns {BaseView}
     */
    BaseView.prototype.disable = function() {
        if (!this.isEnabled) {
            return this;
        }
        this.isEnabled = false;

        this.$pdfButton.off(
            features.TAP,
            this.onPdfClickHandler
        );
        this.$inAppBrowserButton.off(
            features.TAP,
            this.onInAppBrowserClickHandler
        );
        this.$phoneButton.off(
            features.TAP,
            this.onPhoneClickHandler
        );
        this.$addressButton.off(
            features.TAP,
            this.onMapClickHandler
        );
        this.$emailButton.off(
            features.TAP,
            this.onEmailClickHandler
        );
        //filling in for jquery-mobile
        this.$backbutton.off(
            features.TAP,
            this.onBackHandler
        );

        return this;
    };

    BaseView.prototype.viewIsReady = function() {
        var view = this;
        this.performOnNextEventLoop(function() {
            view.trigger(BaseView.VIEW_IS_READY, view);
        });

        return this;
    };

    /**
     * Destroys the component
     * Tears down any events, handlers, elements
     * Should be called when the object should be left unused
     *
     * @returns {BaseView}
     */
    BaseView.prototype.destroy = function() {
        this.disable()
            .removeChildren();

        return this;
    };

    BaseView.prototype.performOnNextEventLoop = function(operation) {
        if (typeof operation !== 'function') {
            throw new TypeError('Cannot perform non-function on next event loop');
        }
        window.setTimeout(operation, 0);

        return this;
    };

    //////////////////////////////////////////////////////////////////////////////////
    // EVENT HANDLERS
    //////////////////////////////////////////////////////////////////////////////////

    /**
     * onInAppBrowserClick handler
     * Handles the click event of a external link button
     *
     * @param {Event} e JavaScript event object
     */
    BaseView.prototype.onInAppBrowserClick = function (event) {
        event.stopImmediatePropagation();
        event.preventDefault();
        var $button = $(event.currentTarget);

        var url = Util.assureHttp($button.data('url'));
        Util.windowOpen(url, '_blank', 'location=no,EnableViewPortScale=yes');
    };

    /**
     * onPdfClick handler
     * Handles the click event of a pdf link button
     *
     * @param {Event} e JavaScript event object
     */
    BaseView.prototype.onPdfClick = function(event) {
        event.preventDefault();
        var $button = $(event.currentTarget);

        var url = $button.data('url');
        if (device.platform == 'Android') {
            url = googleDocsPdfViewerUrl + url;
        }

        Util.windowOpen(url, '_blank', 'location=no,EnableViewPortScale=yes');
    };

    /**
     * onPhoneClick handler
     * Handles the click event of a phone button
     *
     * @param {Event} e JavaScript event object
     */
    BaseView.prototype.onPhoneClick = function(event) {
        event.preventDefault();
        var $button = $(event.currentTarget);
        var phone = Util.stringToNumber($button.data('phone'));

        window.location = 'tel:+1' + phone;
    };

    /**
     * onMapClick handler
     * Handles the click event of a map button
     *
     * @param {Event} e JavaScript event object
     */
    BaseView.prototype.onMapClick = function(event) {
        event.preventDefault();
        var $button = $(event.currentTarget);

        var address = null;
        switch (device.platform) {
            case 'iOS':
                address =
                    'maps:q=' + encodeURIComponent($button.data('address'));
                Util.windowOpen(address);
                break;

            case 'Android':
                address =
                    'geo:0,0?q=' + encodeURIComponent($button.data('address'));
                window.location = address;
                break;
        }
    };

    /**
     * onEmailClick handler
     * Handles the click event of an email button
     *
     * @param {Event} e JavaScript event object
     */
    BaseView.prototype.onEmailClick = function(event) {
        event.preventDefault();
        var $button = $(event.currentTarget);
    };

    //filling in for jquery-mobile
    BaseView.prototype.onBackClick = function (event) {
        //TODO: verify works in iOS
        history.go(-1);
    };

    BaseView.prototype.optionsOnRecieveDataCallback = function() {
        this.$optionsPartial.css('display', 'block');
    };

    BaseView.VIEW_IS_READY = 'viewisready';

    return BaseView;
});
